# -*- coding: utf-8 -*-
import datetime

from django.db import models
from django.conf import settings

from reiare.asamashie import rakuten_access


class AsamashiWord(models.Model):
    OR_FLAG_CHOICES = (
        (0, 'AND検索'),
        (1, 'OR検索')
    )
    keyword = models.CharField("検索語", maxlength=200, unique=True)
    ng_keyword = models.CharField("除外検索語", maxlength=200, null=True, blank=True)
    or_flag = models.IntegerField(default=0, choices=OR_FLAG_CHOICES, help_text='併用不可')
    description = models.TextField("概要", blank=True)
    created = models.DateTimeField("登録日時", editable=False)
    modified = models.DateTimeField("更新日時", editable=False)

    class Admin:
        fields = ((None, {'fields': ('keyword', 'description', 'or_flag')}),)

        date_hierarchy = "modified"
        list_display = ('keyword', 'created', 'modified')
        list_filter = ('modified', 'created')
        search_fields = ['keyword']
        #pass

    class Meta:
        get_latest_by = 'modified'
        ordering = ['-modified']

    def __unicode__(self):
        return self.keyword

    def save(self):
        if not self.id:
            self.created = datetime.datetime.now()

        self.modified = datetime.datetime.now()

        super(AsamashiWord, self).save()

    def get_absolute_url(self):
        return '/asamashie/%s/' % self.id

class AsamashiRakutenXML(models.Model):
    SORT_CHOICES = (
        ('+affiliateRate', 'アフィリエイト料率順（昇順）'),
        ('-affiliateRate', 'アフィリエイト料率順（降順）'),
        ('+reviewCount', 'レビュー件数順（昇順）'),
        ('-reviewCount', 'レビュー件数順（降順）'),
        ('+itemPrice', '価格順（昇順）'),
        ('-itemPrice', '価格順（降順）'),
        ('+updateTimestamp', '商品更新日時順（昇順）'),
        ('-updateTimestamp', '商品更新日時順（降順）'),
        ('random', 'ランダム順')
    )
    xml = models.TextField(editable=False)
    page_count = models.IntegerField(default=1)
    min_price = models.IntegerField(null=True, blank=True)
    max_price = models.IntegerField(null=True, blank=True)
    sort = models.CharField("ソート", maxlength=100, choices=SORT_CHOICES, default='+itemPrice')
    modified = models.DateTimeField("更新日時", editable=False)
    genre_id = models.IntegerField(null=True, blank=True)

    asamashi_word = models.ForeignKey(AsamashiWord, null=True, blank=True)

    def save(self):
        self.modified = datetime.datetime.now()

        super(AsamashiRakutenXML, self).save()

    def get_xml(self):
        if self.xml == '' \
                or self.modified < (datetime.datetime.now() + datetime.timedelta(hours=-12)) \
                or (self.asamashi_word and self.modified < self.asamashi_word.modified) \
                or self.xml == '503 Service Unavailable\n':
            #print "rakuten_access"
            if self.asamashi_word:
                req = rakuten_access.Request('ItemSearch',
                                             settings.RAKUTEN_DEV_ID,
                                             settings.RAKUTEN_AFF_ID,
                                             keyword=self.asamashi_word.keyword,
                                             page=self.page_count,
                                             sort=self.sort,
                                             min_price=self.min_price,
                                             max_price=self.max_price,
                                             genre_id=self.genre_id)
            else:
                req = rakuten_access.Request('ItemSearch',
                                             settings.RAKUTEN_DEV_ID,
                                             settings.RAKUTEN_AFF_ID,
                                             page=self.page_count,
                                             sort=self.sort,
                                             min_price=self.min_price,
                                             max_price=self.max_price,
                                             genre_id=self.genre_id)
            self.xml = rakuten_access.fetch(req)
            self.save()

        return self.xml

    def get_dict(self):
        try:
            res = rakuten_access.Response(self.get_xml())
        except:
            return None

        dict = {'count': res.count,
                'page': int(res.page),
                'first': res.first,
                'last': res.last,
                'hits': res.hits,
                'carrier': res.carrier,
                'page_count': res.page_count,
                'items': res.items,}
                #'page_counts': range(1, int(res.page_count)+1, 1)}
        return dict

class AsamashiRakutenGenreXML(models.Model):
    genre_id = models.IntegerField(unique=True)
    xml = models.TextField(editable=False)
    modified = models.DateTimeField("更新日時", editable=False)

    def save(self):
        self.modified = datetime.datetime.now()

        super(AsamashiRakutenGenreXML, self).save()

    def get_xml(self):
        if self.xml == '' \
                or self.modified < (datetime.datetime.now() + datetime.timedelta(weeks=-1)) \
                or self.xml == '503 Service Unavailable\n':
            req = rakuten_access.Request('GenreSearch',
                                         settings.RAKUTEN_DEV_ID,
                                         settings.RAKUTEN_AFF_ID,
                                         genre_id=self.genre_id)
            self.xml = rakuten_access.fetch(req)
            self.save()

        return self.xml

    def get_dict(self):
        try:
            res = rakuten_access.Response(self.get_xml())
        except:
            return None

        dict = {'parents': res.parents,
                'current_genre_id': res.current_genre_id,
                'current_genre_name': res.current_genre_name,
                'current_genre_level': res.current_genre_level,
                'children': res.children,
                'modified': self.modified}
        return dict
