# -*- coding: utf-8 -*-
import datetime, urllib, os, Image
from xml.etree import ElementTree

from django.db import models
from django.conf import settings
from django.contrib.auth.models import User
from django.core.cache import cache
from django.template import defaultfilters
from django.db.models import permalink


class EntryArchive(models.Model):
    yearmonth = models.CharField(maxlength=6, blank=False)

    class Meta:
        ordering = ['-yearmonth']

    @permalink
    def get_absolute_url(self):
        return ('django.views.generic.date_based.archive_month', (), {
            'year': self.year,
            'month': self.month})

    def __str__(self):
        return self.yearmonth

    def _year(self):
        return self.yearmonth[:4]
    year = property(_year)

    def _month(self):
        return self.yearmonth[4:6]
    month = property(_month)

    def verbose_month(self):
        if self.yearmonth[4:5] == '0':
            return self.yearmonth[5:6]
        else:
            return self.yearmonth[4:6]

    def is_jan(self):
        return self.month == '01'


class EntryTag(models.Model):
    name = models.CharField('名称', maxlength=100, blank=False, unique=True, help_text='重複不可')

    class Meta:
        ordering = ['name']
        verbose_name = 'たぐ'
        verbose_name_plural = 'たぐ'

    class Admin:
        #fields = ((None, {'fields': ('name', self.entry_set)}),)
        pass

    def __str__(self):
        return self.name

    @permalink
    def get_absolute_url(self):
        return ('reiare.blog.views.tag_list', (), {
            'tag': defaultfilters.urlencode(self.name)})

    @permalink
    def get_feeds_url(self):
        return ('django.contrib.syndication.views.feed', (), {
            'url': 'tag/%s' % defaultfilters.urlencode(self.name)})


class PublishedEntryManager(models.Manager):
    def get_query_set(self):
        return super(PublishedEntryManager, self).get_query_set().filter(is_publish=True)

class Entry(models.Model):
    title = models.CharField("タイトル", maxlength=200, blank=False)
    body = models.TextField("本文", blank=False)
    slug = models.SlugField("スラグ", unique_for_date='created', prepopulate_from=('title',))
    created = models.DateTimeField("公開日時")
    updated = models.DateTimeField("変更日時", auto_now=True)
    is_publish = models.BooleanField("公開する", blank=False)
    allow_comment = models.BooleanField("コメントを受け付ける", blank=False, default=True)
    comment_count = models.IntegerField("コメント数", default=0, editable=False)

    created_by = models.ForeignKey(User, related_name='entries', verbose_name="投稿者")
    tags = models.ManyToManyField(EntryTag, filter_interface=models.HORIZONTAL, related_name='entries', verbose_name="タグ", blank=True)
    rel_entries = models.ManyToManyField('self', filter_interface=models.HORIZONTAL, verbose_name="関連エントリ", blank=True, symmetrical=True)

    objects = models.Manager()
    published_objects = PublishedEntryManager()

    class Meta:
        get_latest_by = 'created'
        ordering = ['-created']
        verbose_name_plural = 'えんとりー'
        verbose_name = 'えんとりー'

    class Admin:
        fields = (
            (None, {
                'fields': ('created_by', 'title', 'body', 'slug', 'created', 'is_publish', 'allow_comment'),
            }),
            ('タグ', {
                'fields': ('tags',),
                #'classes': 'collapse'
            }),
            ('関連エントリ', {
                'fields': ('rel_entries',),
                'classes': 'collapse'
            }),
        )
        #js = ['js/admin/AddRichTextEditing.js']

        date_hierarchy = 'created'
        list_display = ('title', 'created', 'updated', 'comment_count', 'is_publish', 'created_by')
        list_filter = ('created_by', 'created', 'is_publish', 'tags')
        list_per_page = 50
        ordering = ['-created', '-updated']
        search_fields = ['title', 'body']
        list_select_related = True

    def __str__(self):
        return self.title

    def save(self):
        super(Entry, self).save()
        EntryArchive.objects.get_or_create(yearmonth='%4d%02d' % (self.created.year, self.created.month))

    @permalink
    def get_absolute_url(self):
        return ('reiare.blog.views.detail', (), {'year': self.created.year,
                                                 'month': self.created.strftime('%m'),
                                                 'day': self.created.strftime('%d'),
                                                 'slug': self.slug})

    def published_comment_list(self):
        return self.comments.filter(is_publish=True).order_by('created')

    def published_comment_count(self):
        if self.comments.filter(is_publish=False):
            return self.comment_count - self.comments.filter(is_publish=False).count()
        else:
            return self.comment_count

    def now_datetime(self):
        return datetime.datetime.now()

    def self_id(self):
        return self.id


class PublishedCommentManager(models.Manager):
	def get_query_set(self):
		return super(PublishedCommentManager, self).get_query_set().filter(is_publish=True)


class Comment(models.Model):
    author = models.CharField("投稿者", maxlength=50, blank=False)
    body = models.TextField("本文", blank=False)
    created = models.DateTimeField("登録日時", auto_now_add=True)
    is_publish = models.BooleanField("公開設定", blank=False, default=True)

    entry = models.ForeignKey(Entry, related_name='comments', verbose_name="エントリー")

    objects = models.Manager()
    published_objects = PublishedCommentManager()

    class Meta:
        get_latest_by = 'created'
        ordering = ['-created']
        #order_with_respect_to = 'entry'
        verbose_name = 'こめんと'
        verbose_name_plural = 'こめんと'

    class Admin:
        fields = ((None, {'fields': ('author', 'body', 'is_publish', 'entry')}),)
        date_hierarchy = ('created')
        list_display = ('created', 'author', 'is_publish', 'entry')
        list_filter = ('created', 'is_publish')
        search_fields = ['author', 'body']
        ordering = ['-created']

    def save(self):
        if self.id == None:
            e = self.entry
            e.comment_count += 1
            e.save()
        cache.delete('recent_comments')
        super(Comment, self).save()

    def delete(self):
        e = self.entry
        e.comment_count -= 1
        e.save()
        cache.delete('recent_comments')
        super(Comment, self).delete()


class PublishedAmazonAsamashiManager(models.Manager):
    def get_query_set(self):
        return super(PublishedAmazonAsamashiManager, self).get_query_set().filter(is_publish=True)


class AmazonAsamashi(models.Model):
    asin = models.CharField("ASIN", blank=False, maxlength=50)
    comment = models.CharField("コメント", maxlength=50, blank=True)
    created = models.DateTimeField("登録日時", auto_now_add=True)
    is_publish = models.BooleanField("トップで紹介", blank=False, default=True)
    priority = models.IntegerField("優先度", blank=False, default=0)
    title = models.CharField("商品名", maxlength=200, blank=True, editable=False)

    objects = models.Manager()
    published_objects = PublishedAmazonAsamashiManager()

    class Meta:
        ordering = ['-created', 'priority']
        verbose_name_plural = 'アマゾンあさまし'
        verbose_name = "アマゾンあさまし"

    class Admin:
        date_hierarchy = ('created')
        list_display = ('title', 'is_publish', 'priority')
        list_filter = ('created', 'is_publish')
        search_fields = ['title']
        ordering = ['-is_publish', '-priority', '-created']

    def __str__(self):
        if self.title == None:
            return "アイテム無し"
        return self.title

    def save(self):
        url = '{http://webservices.amazon.com/AWSECommerceService/2006-11-14}'
        try:
            xml = ElementTree.parse(urllib.urlopen("http://webservices.amazon.co.jp/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + settings.AWS_ACCESS_KEY_ID + "&AssociateTag=" + settings.AWS_ASSOCIATE_TAG + "&Operation=ItemLookup&ItemId=" + self.asin + "&ResponseGroup=ItemAttributes&Version=2006-11-14"))
            self.title = str(xml.find(url + 'Items/' + url + 'Item/' + url + 'ItemAttributes/' + url + 'Title').text)
        except:
            self.title = "アイテム無し"
        cache.delete('asamashies_items')

        super(AmazonAsamashi, self).save()


class EntryImage(models.Model):
    description = models.CharField("概要", maxlength=200, blank=True)
    created = models.DateTimeField("追加日", auto_now_add=True)
    updated = models.DateTimeField("変更日時", auto_now=True)
    image = models.ImageField("オリジナルイメージ", upload_to='images/%Y/%m/%d', height_field='height', width_field='width')
    height = models.PositiveIntegerField(null=True, editable=False)
    width = models.PositiveIntegerField(null=True, editable=False)
    thumb_image = models.CharField(blank=True, maxlength=100, editable=False)
    thumb_height = models.PositiveIntegerField(blank=True, null=True, editable=False)
    thumb_width = models.PositiveIntegerField(blank=True, null=True, editable=False)
    medium_image = models.CharField(blank=True, maxlength=100, editable=False)
    medium_height = models.PositiveIntegerField(blank=True, null=True, editable=False)
    medium_width = models.PositiveIntegerField(blank=True, null=True, editable=False)
    small_image = models.CharField(blank=True, maxlength=100, editable=False)
    small_height = models.PositiveIntegerField(blank=True, null=True, editable=False)
    small_width = models.PositiveIntegerField(blank=True, null=True, editable=False)
    is_publish = models.BooleanField("公開する", blank=False, default=True)

    created_by = models.ForeignKey(User, related_name='entry_images', verbose_name="投稿者")
    tags = models.ManyToManyField(EntryTag, filter_interface=models.HORIZONTAL, related_name='entry_images', verbose_name="タグ", blank=True)
    rel_entry_images = models.ManyToManyField('self', filter_interface=models.HORIZONTAL, verbose_name="関連イメージ", blank=True, symmetrical=True)

    class Meta:
        get_latest_by = 'created'
        ordering = ['-created']
        verbose_name = 'しゃしん'
        verbose_name_plural = 'しゃしん'

    class Admin:
        fields = (
            (None, {
                'fields': ('created_by', 'image', 'description', 'is_publish'),
            }),
            ('タグ', {'fields': ('tags',)}),
            ('関連イメージ', {
                'fields': ('rel_entry_images',),
                'classes': 'collapse'
            }),
        )

        date_hierarchy = 'created'
        list_display = ('__str__', 'created', 'updated', 'is_publish', 'created_by')
        list_filter = ('created_by', 'created', 'is_publish', 'tags')
        ordering = ['-created', '-updated']
        search_fields = ['discription', 'image']
        list_select_related = True

    def __str__(self):
        return self.description + "(" + os.path.basename(self.image) + ")"

    def save(self):
        #if not self.id == None:
        base, ext = os.path.splitext(self.get_image_url())
        medium_image = {"path": base + "_medium" + ext, "width": 500, "height": 375}
        small_image = {"path": base + "_small" + ext, "width": 240, "height": 180}
        thumb_image = {"path": base + "_thumb" + ext, "width": 100, "height": 75}
        images = [medium_image, small_image, thumb_image]

        org_img = Image.open(settings.MEDIA_ROOT + "/" + self.get_image_url()).copy()
        try:
            for image in images:
                img = org_img.copy()
                if img.size[0] > image['width'] or img.size[1] > image['height']:
                    if not image['width'] == 100:
                        if img.size[0] < img.size[1]:
                            img.thumbnail((image['height'], image['width']), Image.ANTIALIAS)
                        else:
                            img.thumbnail((image['width'], image['height']), Image.ANTIALIAS)
                    else:
                        img.thumbnail((image['width'], image['height']), Image.ANTIALIAS)

                img.save(settings.MEDIA_ROOT + "/" + image['path'])
                if image['width'] == 500:
                    self.medium_image = image['path']
                    self.medium_width = img.size[0]
                    self.medium_height = img.size[1]
                elif image['width'] == 240:
                    self.small_image = image['path']
                    self.small_width = img.size[0]
                    self.small_height = img.size[1]
                elif image['width'] == 100:
                    self.thumb_image = image['path']
                    self.thumb_width = img.size[0]
                    self.thumb_height = img.size[1]
        except:
            pass

        super(EntryImage, self).save()

    def delete(self):
        medium_image_path = settings.MEDIA_ROOT + "/" + self.medium_image
        thumb_image_path = settings.MEDIA_ROOT + "/" + self.thumb_image
        small_image_path = setting.MEDIA_ROOT + "/" + self.samll_image

        images = [medium_image_path, small_image_path, thumb_image_path]
        for image in images:
            if os.path.exists(image):
                os.remove(image)

        super(EntryImage, self).delete()
